@@ -3,6 +3,7 @@ from datetime import timedelta
 from flask import current_app
 from flask_json import as_json
 from flask_security import current_user
+import timely_beliefs as tb
 
 from flexmeasures.api.common.responses import (
     invalid_domain,
@@ -15,8 +16,8 @@ from flexmeasures.api.common.responses import (
     ResponseTuple,
 )
 from flexmeasures.api.common.utils.api_utils import (
-    get_weather_sensor_by,
-    save_to_db,
+    get_sensor_by_generic_asset_type_and_location,
+    save_and_enqueue,
     determine_belief_timing,
 )
 from flexmeasures.api.common.utils.validators import (
@@ -31,12 +32,10 @@ from flexmeasures.api.common.utils.validators import (
     period_required,
     values_required,
 )
-from flexmeasures.data.models.assets import Asset, Power
-from flexmeasures.data.models.data_sources import get_or_create_source
-from flexmeasures.data.models.markets import Market, Price
-from flexmeasures.data.models.weather import Weather
+from flexmeasures.data.queries.data_sources import get_or_create_source
+from flexmeasures.data.models.time_series import Sensor, TimedBelief
 from flexmeasures.data.services.forecasting import create_forecasting_jobs
-from flexmeasures.data.services.resources import get_assets
+from flexmeasures.data.services.resources import get_sensors
 from flexmeasures.utils.entity_address_utils import (
     parse_entity_address,
     EntityAddressException,
@@ -50,7 +49,7 @@ from flexmeasures.utils.entity_address_utils import (
 @optional_prior_accepted(infer_missing=True, infer_missing_play=False)
 @values_required
 @period_required
-@post_data_checked_for_required_resolution("market")
+@post_data_checked_for_required_resolution("market", "fm1")
 def post_price_data_response(  # noqa C901
     unit,
     generic_asset_name_groups,
@@ -70,7 +69,7 @@ def post_price_data_response(  # noqa C901
     current_app.logger.info("POSTING PRICE DATA")
 
     data_source = get_or_create_source(current_user)
-    prices = []
+    price_df_per_market = []
     forecasting_jobs = []
     for market_group, event_values in zip(generic_asset_name_groups, value_groups):
         for market in market_group:
@@ -80,61 +79,59 @@ def post_price_data_response(  # noqa C901
                 ea = parse_entity_address(market, entity_type="market")
             except EntityAddressException as eae:
                 return invalid_domain(str(eae))
-            market_name = ea["market_name"]
+            sensor_id = ea["sensor_id"]
 
-            # Look for the Market object
-            market = Market.query.filter(Market.name == market_name).one_or_none()
-            if market is None:
-                return unrecognized_market(market_name)
-            elif unit != market.unit:
-                return invalid_unit("%s prices" % market.display_name, [market.unit])
+            # Look for the Sensor object
+            sensor = Sensor.query.filter(Sensor.id == sensor_id).one_or_none()
+            if sensor is None:
+                return unrecognized_market(sensor_id)
+            elif unit != sensor.unit:
+                return invalid_unit("%s prices" % sensor.name, [sensor.unit])
 
             # Convert to timely-beliefs terminology
             event_starts, belief_horizons = determine_belief_timing(
-                event_values, start, resolution, horizon, prior, market
+                event_values, start, resolution, horizon, prior, sensor
             )
 
             # Create new Price objects
-            prices.extend(
-                [
-                    Price(
-                        datetime=event_start,
-                        value=event_value,
-                        horizon=belief_horizon,
-                        market_id=market.id,
-                        data_source_id=data_source.id,
-                    )
-                    for event_start, event_value, belief_horizon in zip(
-                        event_starts, event_values, belief_horizons
-                    )
-                ]
-            )
+            beliefs = [
+                TimedBelief(
+                    event_start=event_start,
+                    event_value=event_value,
+                    belief_horizon=belief_horizon,
+                    sensor=sensor,
+                    source=data_source,
+                )
+                for event_start, event_value, belief_horizon in zip(
+                    event_starts, event_values, belief_horizons
+                )
+            ]
+            price_df_per_market.append(tb.BeliefsDataFrame(beliefs))
 
             # Make forecasts, but not in play mode. Price forecasts (horizon>0) can still lead to other price forecasts,
             # by the way, due to things like day-ahead markets.
             if current_app.config.get("FLEXMEASURES_MODE", "") != "play":
                 # Forecast 24 and 48 hours ahead for at most the last 24 hours of posted price data
                 forecasting_jobs = create_forecasting_jobs(
-                    "Price",
-                    market.id,
+                    sensor.id,
                     max(start, start + duration - timedelta(hours=24)),
                     start + duration,
                     resolution=duration / len(event_values),
                     horizons=[timedelta(hours=24), timedelta(hours=48)],
-                    enqueue=False,  # will enqueue later, only if we successfully saved prices
+                    enqueue=False,  # will enqueue later, after saving data
                 )
 
-    return save_to_db(prices, forecasting_jobs)
+    return save_and_enqueue(price_df_per_market, forecasting_jobs)
 
 
 @type_accepted("PostWeatherDataRequest")
 @unit_required
-@assets_required("sensor")
+@assets_required("weather_sensor")
 @optional_horizon_accepted(infer_missing=False, infer_missing_play=True)
 @optional_prior_accepted(infer_missing=True, infer_missing_play=False)
 @values_required
 @period_required
-@post_data_checked_for_required_resolution("sensor")
+@post_data_checked_for_required_resolution("weather_sensor", "fm1")
 def post_weather_data_response(  # noqa: C901
     unit,
     generic_asset_name_groups,
@@ -153,14 +150,14 @@ def post_weather_data_response(  # noqa: C901
     current_app.logger.info("POSTING WEATHER DATA")
 
     data_source = get_or_create_source(current_user)
-    weather_measurements = []
+    weather_df_per_sensor = []
     forecasting_jobs = []
     for sensor_group, event_values in zip(generic_asset_name_groups, value_groups):
         for sensor in sensor_group:
 
             # Parse the entity address
             try:
-                ea = parse_entity_address(sensor, entity_type="sensor")
+                ea = parse_entity_address(sensor, entity_type="weather_sensor")
             except EntityAddressException as eae:
                 return invalid_domain(str(eae))
             weather_sensor_type_name = ea["weather_sensor_type_name"]
@@ -172,30 +169,29 @@ def post_weather_data_response(  # noqa: C901
             if unit not in accepted_units:
                 return invalid_unit(weather_sensor_type_name, accepted_units)
 
-            weather_sensor = get_weather_sensor_by(
+            sensor: Sensor = get_sensor_by_generic_asset_type_and_location(
                 weather_sensor_type_name, latitude, longitude
             )
 
             # Convert to timely-beliefs terminology
             event_starts, belief_horizons = determine_belief_timing(
-                event_values, start, resolution, horizon, prior, weather_sensor
+                event_values, start, resolution, horizon, prior, sensor
             )
 
             # Create new Weather objects
-            weather_measurements.extend(
-                [
-                    Weather(
-                        datetime=event_start,
-                        value=event_value,
-                        horizon=belief_horizon,
-                        sensor_id=weather_sensor.id,
-                        data_source_id=data_source.id,
-                    )
-                    for event_start, event_value, belief_horizon in zip(
-                        event_starts, event_values, belief_horizons
-                    )
-                ]
-            )
+            beliefs = [
+                TimedBelief(
+                    event_start=event_start,
+                    event_value=event_value,
+                    belief_horizon=belief_horizon,
+                    sensor=sensor,
+                    source=data_source,
+                )
+                for event_start, event_value, belief_horizon in zip(
+                    event_starts, event_values, belief_horizons
+                )
+            ]
+            weather_df_per_sensor.append(tb.BeliefsDataFrame(beliefs))
 
             # make forecasts, but only if the sent-in values are not forecasts themselves (and also not in play)
             if current_app.config.get(
@@ -205,17 +201,16 @@ def post_weather_data_response(  # noqa: C901
             ):  # Todo: replace 0 hours with whatever the moment of switching from ex-ante to ex-post is for this generic asset
                 forecasting_jobs.extend(
                     create_forecasting_jobs(
-                        "Weather",
-                        weather_sensor.id,
+                        sensor.id,
                         start,
                         start + duration,
                         resolution=duration / len(event_values),
                         horizons=[horizon],
-                        enqueue=False,  # will enqueue later, only if we successfully saved weather measurements
+                        enqueue=False,  # will enqueue later, after saving data
                     )
                 )
 
-    return save_to_db(weather_measurements, forecasting_jobs)
+    return save_and_enqueue(weather_df_per_sensor, forecasting_jobs)
 
 
 @type_accepted("PostMeterDataRequest")
@@ -225,7 +220,7 @@ def post_weather_data_response(  # noqa: C901
 @optional_horizon_accepted(ex_post=True, infer_missing=False, infer_missing_play=True)
 @optional_prior_accepted(ex_post=True, infer_missing=True, infer_missing_play=False)
 @period_required
-@post_data_checked_for_required_resolution("connection")
+@post_data_checked_for_required_resolution("connection", "fm1")
 @as_json
 def post_meter_data_response(
     unit,
@@ -257,7 +252,7 @@ def post_meter_data_response(
 @optional_horizon_accepted(ex_post=False, infer_missing=False, infer_missing_play=False)
 @optional_prior_accepted(ex_post=False, infer_missing=True, infer_missing_play=False)
 @period_required
-@post_data_checked_for_required_resolution("connection")
+@post_data_checked_for_required_resolution("connection", "fm1")
 @as_json
 def post_prognosis_response(
     unit,
@@ -302,11 +297,11 @@ def post_power_data(
     current_app.logger.info("POSTING POWER DATA")
 
     data_source = get_or_create_source(current_user)
-    user_assets = get_assets()
-    if not user_assets:
+    user_sensors = get_sensors()
+    if not user_sensors:
         current_app.logger.info("User doesn't seem to have any assets")
-    user_asset_ids = [asset.id for asset in user_assets]
-    power_measurements = []
+    user_sensor_ids = [sensor.id for sensor in user_sensors]
+    power_df_per_connection = []
     forecasting_jobs = []
     for connection_group, event_values in zip(generic_asset_name_groups, value_groups):
         for connection in connection_group:
@@ -314,64 +309,66 @@ def post_power_data(
             # TODO: get asset through util function after refactoring
             # Parse the entity address
             try:
-                connection = parse_entity_address(connection, entity_type="connection")
+                ea = parse_entity_address(connection, entity_type="connection")
             except EntityAddressException as eae:
                 return invalid_domain(str(eae))
-            asset_id = connection["asset_id"]
+            sensor_id = ea["sensor_id"]
 
-            # Look for the Asset object
-            if asset_id in user_asset_ids:
-                asset = Asset.query.filter(Asset.id == asset_id).one_or_none()
+            # Look for the Sensor object
+            if sensor_id in user_sensor_ids:
+                sensor = Sensor.query.filter(Sensor.id == sensor_id).one_or_none()
             else:
                 current_app.logger.warning("Cannot identify connection %s" % connection)
                 return unrecognized_connection_group()
 
             # Validate the sign of the values (following USEF specs with positive consumption and negative production)
-            if asset.is_pure_consumer and any(v < 0 for v in event_values):
+            if sensor.get_attribute("is_strictly_non_positive") and any(
+                v < 0 for v in event_values
+            ):
                 extra_info = (
                     "Connection %s is registered as a pure consumer and can only receive non-negative values."
-                    % asset.entity_address
+                    % sensor.entity_address
                 )
                 return power_value_too_small(extra_info)
-            elif asset.is_pure_producer and any(v > 0 for v in event_values):
+            elif sensor.get_attribute("is_strictly_non_negative") and any(
+                v > 0 for v in event_values
+            ):
                 extra_info = (
                     "Connection %s is registered as a pure producer and can only receive non-positive values."
-                    % asset.entity_address
+                    % sensor.entity_address
                 )
                 return power_value_too_big(extra_info)
 
             # Convert to timely-beliefs terminology
             event_starts, belief_horizons = determine_belief_timing(
-                event_values, start, resolution, horizon, prior, asset
+                event_values, start, resolution, horizon, prior, sensor
             )
 
             # Create new Power objects
-            power_measurements.extend(
-                [
-                    Power(
-                        datetime=event_start,
-                        value=event_value
-                        * -1,  # Reverse sign for FlexMeasures specs with positive production and negative consumption
-                        horizon=belief_horizon,
-                        asset_id=asset.id,
-                        data_source_id=data_source.id,
-                    )
-                    for event_start, event_value, belief_horizon in zip(
-                        event_starts, event_values, belief_horizons
-                    )
-                ]
-            )
+            beliefs = [
+                TimedBelief(
+                    event_start=event_start,
+                    event_value=event_value
+                    * -1,  # Reverse sign for FlexMeasures specs with positive production and negative consumption
+                    belief_horizon=belief_horizon,
+                    sensor=sensor,
+                    source=data_source,
+                )
+                for event_start, event_value, belief_horizon in zip(
+                    event_starts, event_values, belief_horizons
+                )
+            ]
+            power_df_per_connection.append(tb.BeliefsDataFrame(beliefs))
 
             if create_forecasting_jobs_too:
                 forecasting_jobs.extend(
                     create_forecasting_jobs(
-                        "Power",
-                        asset_id,
+                        sensor_id,
                         start,
                         start + duration,
                         resolution=duration / len(event_values),
-                        enqueue=False,  # will enqueue later, only if we successfully saved power measurements
+                        enqueue=False,  # will enqueue later, after saving data
                     )
                 )
 
-    return save_to_db(power_measurements, forecasting_jobs)
+    return save_and_enqueue(power_df_per_connection, forecasting_jobs)
