@@ -3,7 +3,7 @@ from datetime import timedelta, datetime
 import pytest
 from sqlalchemy.orm import Query
 
-from flexmeasures.data.models.assets import Asset, Power
+from flexmeasures.data.models.time_series import Sensor, TimedBelief
 from flexmeasures.data.services.forecasting import (
     create_forecasting_jobs,
     handle_forecasting_exception,
@@ -18,17 +18,18 @@ from flexmeasures.data.tests.utils import work_on_rq
 from flexmeasures.utils.time_utils import as_server_time
 
 
-def test_forecasting_three_hours_of_wind(app, setup_fresh_test_data, clean_redis):
-    wind_device2: Asset = Asset.query.filter_by(name="wind-asset-2").one_or_none()
+def test_forecasting_three_hours_of_wind(
+    app, run_as_cli, setup_fresh_test_data, clean_redis
+):
+    wind_device2: Sensor = Sensor.query.filter_by(name="wind-asset-2").one_or_none()
 
     # makes 12 forecasts
     horizon = timedelta(hours=1)
     job = create_forecasting_jobs(
-        timed_value_type="Power",
         start_of_roll=as_server_time(datetime(2015, 1, 1, 10)),
         end_of_roll=as_server_time(datetime(2015, 1, 1, 13)),
         horizons=[horizon],
-        asset_id=wind_device2.id,
+        sensor_id=wind_device2.id,
         custom_model_params=custom_model_params(),
     )
     print("Job: %s" % job[0].id)
@@ -36,11 +37,11 @@ def test_forecasting_three_hours_of_wind(app, setup_fresh_test_data, clean_redis
     work_on_rq(app.queues["forecasting"], exc_handler=handle_forecasting_exception)
 
     forecasts = (
-        Power.query.filter(Power.asset_id == wind_device2.id)
-        .filter(Power.horizon == horizon)
+        TimedBelief.query.filter(TimedBelief.sensor_id == wind_device2.id)
+        .filter(TimedBelief.belief_horizon == horizon)
         .filter(
-            (Power.datetime >= as_server_time(datetime(2015, 1, 1, 11)))
-            & (Power.datetime < as_server_time(datetime(2015, 1, 1, 14)))
+            (TimedBelief.event_start >= as_server_time(datetime(2015, 1, 1, 11)))
+            & (TimedBelief.event_start < as_server_time(datetime(2015, 1, 1, 14)))
         )
         .all()
     )
@@ -48,31 +49,32 @@ def test_forecasting_three_hours_of_wind(app, setup_fresh_test_data, clean_redis
     check_aggregate(12, horizon, wind_device2.id)
 
 
-def test_forecasting_two_hours_of_solar(app, setup_fresh_test_data, clean_redis):
-    solar_device1: Asset = Asset.query.filter_by(name="solar-asset-1").one_or_none()
-    wind_device2: Asset = Asset.query.filter_by(name="wind-asset-2").one_or_none()
+def test_forecasting_two_hours_of_solar(
+    app, run_as_cli, setup_fresh_test_data, clean_redis
+):
+    solar_device1: Sensor = Sensor.query.filter_by(name="solar-asset-1").one_or_none()
+    wind_device2: Sensor = Sensor.query.filter_by(name="wind-asset-2").one_or_none()
     print(solar_device1)
     print(wind_device2)
 
     # makes 8 forecasts
     horizon = timedelta(hours=1)
     job = create_forecasting_jobs(
-        timed_value_type="Power",
         start_of_roll=as_server_time(datetime(2015, 1, 1, 12)),
         end_of_roll=as_server_time(datetime(2015, 1, 1, 14)),
         horizons=[horizon],
-        asset_id=solar_device1.id,
+        sensor_id=solar_device1.id,
         custom_model_params=custom_model_params(),
     )
     print("Job: %s" % job[0].id)
 
     work_on_rq(app.queues["forecasting"], exc_handler=handle_forecasting_exception)
     forecasts = (
-        Power.query.filter(Power.asset_id == solar_device1.id)
-        .filter(Power.horizon == horizon)
+        TimedBelief.query.filter(TimedBelief.sensor_id == solar_device1.id)
+        .filter(TimedBelief.belief_horizon == horizon)
         .filter(
-            (Power.datetime >= as_server_time(datetime(2015, 1, 1, 13)))
-            & (Power.datetime < as_server_time(datetime(2015, 1, 1, 15)))
+            (TimedBelief.event_start >= as_server_time(datetime(2015, 1, 1, 13)))
+            & (TimedBelief.event_start < as_server_time(datetime(2015, 1, 1, 15)))
         )
         .all()
     )
@@ -84,7 +86,12 @@ def test_forecasting_two_hours_of_solar(app, setup_fresh_test_data, clean_redis)
     "model_to_start_with, model_version", [("failing-test", 1), ("linear-OLS", 2)]
 )
 def test_failed_model_with_too_much_training_then_succeed_with_fallback(
-    setup_fresh_test_data, app, clean_redis, model_to_start_with, model_version
+    app,
+    run_as_cli,
+    clean_redis,
+    setup_fresh_test_data,
+    model_to_start_with,
+    model_version,
 ):
     """
     Here we fail once - because we start with a model that needs too much training.
@@ -93,7 +100,7 @@ def test_failed_model_with_too_much_training_then_succeed_with_fallback(
     (fail-test falls back to linear & linear falls back to naive).
     As a result, there should be forecasts in the DB.
     """
-    solar_device1: Asset = Asset.query.filter_by(name="solar-asset-1").one_or_none()
+    solar_device1: Sensor = Sensor.query.filter_by(name="solar-asset-1").one_or_none()
     horizon_hours = 1
     horizon = timedelta(hours=horizon_hours)
 
@@ -105,11 +112,10 @@ def test_failed_model_with_too_much_training_then_succeed_with_fallback(
 
     # The failed test model (this failure enqueues a new job)
     create_forecasting_jobs(
-        timed_value_type="Power",
         start_of_roll=as_server_time(datetime(2015, 1, 1, hour_start)),
         end_of_roll=as_server_time(datetime(2015, 1, 1, hour_start + 2)),
         horizons=[horizon],
-        asset_id=solar_device1.id,
+        sensor_id=solar_device1.id,
         model_search_term=model_to_start_with,
         custom_model_params=cmp,
     )
@@ -126,17 +132,17 @@ def test_failed_model_with_too_much_training_then_succeed_with_fallback(
     def make_query(the_horizon_hours: int) -> Query:
         the_horizon = timedelta(hours=the_horizon_hours)
         return (
-            Power.query.filter(Power.asset_id == solar_device1.id)
-            .filter(Power.horizon == the_horizon)
+            TimedBelief.query.filter(TimedBelief.sensor_id == solar_device1.id)
+            .filter(TimedBelief.belief_horizon == the_horizon)
             .filter(
                 (
-                    Power.datetime
+                    TimedBelief.event_start
                     >= as_server_time(
                         datetime(2015, 1, 1, hour_start + the_horizon_hours)
                     )
                 )
                 & (
-                    Power.datetime
+                    TimedBelief.event_start
                     < as_server_time(
                         datetime(2015, 1, 1, hour_start + the_horizon_hours + 2)
                     )
@@ -154,7 +160,7 @@ def test_failed_model_with_too_much_training_then_succeed_with_fallback(
         existing_data = make_query(the_horizon_hours=0).all()
 
         for ed, fd in zip(existing_data, forecasts):
-            assert ed.value == fd.value
+            assert ed.event_value == fd.event_value
 
     # Now to check which models actually got to work.
     # We check which data sources do and do not exist by now:
